var playerState = {
    IDLE : 0,
    IDLENOTELEPORT : 1,
    WALKING : 2,
    WALKINGNOTELEPORT : 3,
//    JUMPING : 4,
//    JUMPINGNOTELEPORT : 5,
    SPAWNING : 4,
    DYING : 5,
    DYINGNOTELEPORT : 6,
    DEAD : 7,
    RESPAWNING : 8
};

Player = function(game, x, y, tagP, tagT) {
    Phaser.Sprite.call(this, game, x, y, tagP);
    this.game = game;
    this.anchor.setTo(0.5, 1.0);
    this.state = playerState.IDLE;
    this.walkingVelocity = 300;
    this.jumpingVelocity = 500;
    this.maxFallingVelocity = 400;
    this.teleporter = 0;
    this.teleporterTimeToLive = 5000; //milliseconds
    this.recentlySpawned = false;
    this.canCheckRecentSpawn = true;
    this.tagT = tagT;
    this.lookingLeft = false;
    this.teleporterWasShot = false;
    this.didJustLand = false;

    this.timer = new Phaser.Timer();
    this.createAnimations();
    this.createTeleporter(0,0);
    this.setStateIsIdle();
}

Player.prototype = Object.create(Phaser.Sprite.prototype);
Player.prototype.constructor = Player;

Player.prototype.createTeleporter = function(mx, my) {

    if (this.teleporter === 0)
    {
        this.teleporter = new Teleporter(this.game, this.x, this.y - this.height/2, this.tagT);
        this.game.add.existing(this.teleporter);
//        this.teleporter.reset((mx > this.x ? this.x + this.width/2 : this.x - this.width/2), this.y - this.height/2, mx, my);
    }
    else
    {
        this.teleporter.reset(this.x, this.y - this.height/2, mx, my);
        this.teleporterWasShot = true;
    }
}

Player.prototype.createAnimations = function()
{
    if(this.key == "white")
    {
        animIdle                  = this.animations.add('idle', [27,28,29,30,31,32,33,34,35,36,37,38,39], 20, true);
        animIdleNoTeleport        = this.animations.add('idleNoTeleport', [40,41,42,43,44,45,46,47,48,49,50,51], 15, true);

        this.animations.add('walking', [52,53,54,55,56,57,58,59,60,61,62,63,64,65], 20, true);
        this.animations.add('walkingNoTeleport', [66,67,68,69,70,71,72,73,74,75,76,77,78,79], 20, true);

        animDying = this.animations.add('dying', [1,2,3,4,5,6,7,8,9,10,11,12,13,14], 20, false);
        animDyingNoTeleport = this.animations.add('dyingNoTeleport', [14,15,16,17,18,19,20,21,22,23,24,25,26], 20, false);

        this.animations.play('idle');
    }
    else
    {
        animIdle                  = this.animations.add('idle', [16,17,18,19,20,21,22,23,24,25,26,27,28,29,30], 20, true);
        animIdleNoTeleport        = this.animations.add('idleNoTeleport', [31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46], 15, true);

        this.animations.add('walking', [47,48,49,50,51,52,53,54,55,56,57,58,59,60,61], 20, true);
        this.animations.add('walkingNoTeleport', [62,63,64,65,66,67,68,69,70,71,72,73,74,75,76], 20, true);

        animDying = this.animations.add('dying', [1,2,3,4,5,6,7], 20, false);
        animDyingNoTeleport = this.animations.add('dyingNoTeleport', [8,9,10,11,12,13,14,15], 20, false);

        this.animations.play('idle');
    }
}

Player.prototype.spawnAt = function(x, y) {
    this.x = x;
    this.y = y;
    this.body.velocity.x = 0;
    this.body.velocity.y = 0;
    this.recentlySpawned = true;
    this.canCheckRecentSpawn = false;
    this.game.time.events.add(Phaser.Timer.SECOND * 0.05, this.allowRecentSpawnCheck, this);
    console.log('Will spawn next frame!');
}

Player.prototype.walkRight = function() {
    this.body.velocity.x = this.walkingVelocity;

    if(this.state != playerState.WALKING && this.state != playerState.WALKINGNOTELEPORT)
    {
        if(this.teleporterWasShot)
        {
            this.animations.play('walkingNoTeleport');
            this.state = playerState.WALKINGNOTELEPORT;
//            animWalking.play(15, true);
        }
        else
        {
            this.animations.play('walking');
            this.state = playerState.WALKING;
//            animWalkingNoTeleport.play(15, true);
        }
    }

    console.log("State: " + this.state);

    if(this.lookingLeft)
    {
        this.scale.x = 0.6;
        this.lookingLeft = false;
    }
}

Player.prototype.walkLeft = function() {
    this.body.velocity.x = -this.walkingVelocity;

    if(this.state != playerState.WALKING && this.state != playerState.WALKINGNOTELEPORT)
    {
//        this.tint = 0x000000;
        //play animation
        if(this.teleporterWasShot)
        {
            this.animations.play('walkingNoTeleport');
        }
        else
        {
            this.animations.play('walking');
        }
    }

    if(this.teleporterWasShot)
        this.state = playerState.WALKINGNOTELEPORT;
    else
        this.state = playerState.WALKING;

    console.log("State: " + this.state);

    if(!this.lookingLeft)
    {
        this.scale.x = -0.6;
        this.lookingLeft = true;
    }
}

Player.prototype.canJump = function() {
//    return this.body.onFloor();
    return this.body.blocked.down || this.body.touching.down;
}

Player.prototype.jump = function() {
    this.body.velocity.y = -this.jumpingVelocity;
    this.didJustLand = false;
}

Player.prototype.stopWalking = function() {
    this.body.velocity.x = 0;
    if(this.state != playerState.IDLE && this.state != playerState.IDLENOTELEPORT)
        this.setStateIsIdle();
}

Player.prototype.resetHorizontalVelocity = function() {
    this.body.velocity.x = 0;
//    this.setStateIsIdle();
}

Player.prototype.resetVerticalVelocity = function() {
    this.body.velocity.y = 0;
//    this.setStateIsIdle();
}

Player.prototype.update = function()
{

//    if(this.body.velocity.x == 0 && this.body.velocity.y == 0)
//        this.state = playerState.IDLE;

//    if(animIdle.isPlaying)
//        console.log("Playing Idle");
//    else
//        console.log("NOT Playing Idle");

    if(this.teleporterWasShot && this.state === playerState.IDLE)
    {
        this.setStateIsIdle();
    }
    else if(!this.teleporterWasShot && this.state === playerState.IDLENOTELEPORT)
    {
        this.setStateIsIdle();
    }


    if (this.canCheckRecentSpawn)
    {
        if (this.body.touching.down)
        {
            if(this.recentlySpawned)
                this.teleporterWasShot = false;

            this.recentlySpawned = false;
        }
    }

    if(this.body.touching.down && !this.didJustLand)
    {
        this.didJustLand = true;
    }

    //This is to avoid the player falling too fast and hitting the other player too hard
    if(this.body.velocity.y > this.maxFallingVelocity)
    {
        this.body.velocity.y = this.maxFallingVelocity
    }
}

Player.prototype.setStateIsRespawning = function()
{
    this.state = playerState.RESPAWNING;
    console.log("State: " + this.state);
}

Player.prototype.isRespawning = function()
{
    return this.state === playerState.RESPAWNING;
}

Player.prototype.setStateDying = function()
{
    if(this.teleporterWasShot)
    {
        this.state = playerState.DYINGNOTELEPORT;
        console.log("State: " + this.state);

        animDyingNoTeleport = this.animations.play('dyingNoTeleport');
        animDyingNoTeleport.onComplete.add(this.animDyingCompleted, this);
    }
    else
    {
        this.state = playerState.DYING;
        console.log("State: " + this.state);

        animDying = this.animations.play('dying');
        animDying.onComplete.add(this.animDyingCompleted, this);
    }

    this.teleporter.spawnPlayer();
}

Player.prototype.setStateIsIdle = function()
{
    if(this.state === playerState.DYING || this.state === playerState.DYINGNOTELEPORT)// || this.state === playerState.RESPAWNING)
        return;

    if(this.teleporterWasShot)
    {
        this.animations.play('idleNoTeleport');
    }
    else{
        this.animations.play('idle');
    }


    if(this.teleporterWasShot)
    {
        this.state = playerState.IDLENOTELEPORT;
        console.log("State: " + this.state);
    }
    else
    {
        this.state = playerState.IDLE;
        console.log("State: " + this.state);
    }
}

Player.prototype.isIdle = function()
{
    return this.state === playerState.IDLE || this.state === playerState.IDLENOTELEPORT;
}

Player.prototype.isDead = function()
{
    return this.state === playerState.DEAD;
}

Player.prototype.isDying = function()
{
    return this.state === playerState.DYING || this.state === playerState.DYINGNOTELEPORT;
}

Player.prototype.animDyingCompleted = function()
{
    this.teleporterWasShot = false;
    this.state = playerState.DEAD;
}

Player.prototype.hasTeleporter = function() {
    return !(this.teleporter === 0);
}

Player.prototype.allowRecentSpawnCheck = function()
{
    this.canCheckRecentSpawn = true;
}
